;; This work is licensed under the Creative Commons
;; Attribution-NonCommercial-ShareAlike License. To view a copy of this
;; license, visit http://creativecommons.org/licenses/by-nc-sa/2.0/ or
;; send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford,
;; California 94305, USA.
;;
;; The original author of this code is Arthur Nunes-Harwitt
;;
;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;; The Evaluator
;;;;;;;;;;;;;;;;;;;;;;;;;


(define *proc-return* '(no value))

(define *logo-true* (make-name "TRUE"))

(define *logo-false* (make-name "FALSE"))

(define (logo-true? x)
  (and (logo-name? x)
       (eq? 'true (symbol<-logo-name x))))

(define (logo-false? x)
  (and (logo-name? x)
       (eq? 'false (symbol<-logo-name x))))

(define (logo-boolean? x)
  (and (logo-name? x)
       (or (eq? 'false (symbol<-logo-name x))
           (eq? 'true (symbol<-logo-name x)))))

(define (logo-bool<-bool x)
  (case x
    ((#t) *logo-true*)
    ((#f) *logo-false*)
    (else (logo-error "Not a boolean value" x))))

(define (bool<-logo-bool x)
  (case (symbol<-logo-name x)
    ((true) #t)
    ((false) #f)
    (else (logo-error "Not a Logo boolean value" x))))

(define (self-eval? exp)  (or (number? exp) 
                              (logo-list? exp))) 

(define (look-up name env)
  (if (null? env)
    (logo-error "unbound variable" name)
    (let ((names (car env))
          (vals (cadr env))
          (rest-env (cddr env)))
      (let loop ((names names)
                  (vals vals))
        (cond ((null? names)
               (look-up name rest-env))
              ((eq? name (car names))
               (car vals))
              (else (loop (cdr names) (cdr vals))))))))


(define (v-look-up name env)
  (if (null? env)
    (logo-error "The variable" name "has no value")
    (let ((names (car env))
          (vals (cadr env))
          (rest-env (cddr env)))
      (let loop ((names names)
                  (vals vals))
        (cond ((null? names)
               (v-look-up name rest-env))
              ((eq? name (car names))
               (car vals))
              (else (loop (cdr names) (cdr vals))))))))


(define (p-look-up name env)
  (if (null? env)
    (logo-error "I don't know how to" name)
    (let ((names (car env))
          (vals (cadr env))
          (rest-env (cddr env)))
      (let loop ((names names)
                  (vals vals))
        (cond ((null? names)
               (p-look-up name rest-env))
              ((eq? name (car names))
               (car vals))
              (else (loop (cdr names) (cdr vals))))))))


(define (c-look-up name env)
  (if (null? env)
    (logo-error "I can't find the catch tag" name)
    (let ((names (car env))
          (vals (cadr env))
          (rest-env (cddr env)))
      (let loop ((names names)
                  (vals vals))
        (cond ((null? names)
               (c-look-up name rest-env))
              ((eq? name (car names))
               (car vals))
              (else (loop (cdr names) (cdr vals))))))))

(define (extend env names vals)
  (cons names (cons vals env)))

(define (local-update! env name val)
  (let ((names (car env))
        (vals (cadr env))
        (rest-env (cddr env)))
    (let loop ((nmes names)
               (vls vals))
      (cond ((null? nmes)
             (set-car! env (cons name names))
             (set-car! (cdr env) (cons val vals)))
            ((eq? name (car nmes))
             (set-car! vls val))
            (else (loop (cdr nmes) (cdr vls)))))))

(define (update! env name val)
  (let ((names (car env))
        (vals (cadr env))
        (rest-env (cddr env)))
    (if (null? rest-env)
      (let loop ((nmes names)
                 (vls vals))
      (cond ((null? nmes)
             (set-car! env (cons name names))
             (set-car! (cdr env) (cons val vals)))
            ((eq? name (car nmes))
             (set-car! vls val))
            (else (loop (cdr nmes) (cdr vls)))))
      (let loop ((nmes names)
                 (vls vals))
        (cond ((null? nmes) (update! rest-env name val))
              ((eq? name (car nmes))
               (set-car! vls val))
              (else (loop (cdr nmes) (cdr vls))))))))


(define (make-primitive prim)
  (vector 'primitive prim))
(define (logo-primitive? x)
  (and (vector? x) (eq? (vector-ref x 0) 'primitive)))
(define (code<-primitive x) (vector-ref x 1))

(define (make-user-proc code rep)
  (vector 'user-proc code rep))
(define (logo-user-proc? x)
  (and (vector? x) (eq? (vector-ref x 0) 'user-proc)))
(define (code<-user-proc x) (vector-ref x 1))
(define (rep<-user-proc x) (vector-ref x 2))
(define (set-user-proc-code! up c) (vector-set! up 1 c))
(define (set-user-proc-rep! up r) (vector-set! up 2 r))


(define (code<-logo-proc-entry x) (vector-ref x 1))


(define *init-var-env* (extend '() '() '()))

(define *init-catch-env* '())

(define (elog exp v-env p-env c-env k-s k-ret)
  (cond ((stop? exp) 
	 (k-ret *proc-return*))
        ((output? exp) 
	 (elog-output exp v-env p-env c-env k-s k-ret))
        ((self-eval? exp) 
	 (k-s exp))
        ((logo-literal-word? exp)
	 (elog-logo-literal-word exp v-env p-env c-env k-s k-ret))
        ((logo-var? exp) 
	 (k-s (v-look-up (symbol<-logo-var exp) v-env)))
        ((empty-seq? exp) 
	 (k-s *proc-return*))
        ((seq? exp)
	 (elog-seq exp v-env p-env c-env k-s k-ret))
        ((catch? exp)
	 (elog-catch exp v-env p-env c-env k-s k-ret))
        ((throw? exp)
	 (elog-throw exp v-env p-env c-env k-s k-ret))
        ((if? exp)
	 (elog-if exp v-env p-env c-env k-s k-ret))
        ((ifelse? exp)
	 (elog-ifelse exp v-env p-env c-env k-s k-ret))
        ((repeat? exp) 
	 (elog-repeat exp v-env p-env c-env k-s k-ret))
        ((make? exp)
	 (elog-make exp v-env p-env c-env k-s k-ret))
        ((proc? exp) 
	 (elog-proc exp v-env p-env c-env k-s k-ret))
	((sum? exp)
	 (elog-sum exp v-env p-env c-env k-s k-ret))
        ((diff? exp)
	 (elog-diff exp v-env p-env c-env k-s k-ret))
        ((prod? exp)
	 (elog-prod exp v-env p-env c-env k-s k-ret))
        ((quo? exp) 
	 (elog-quo exp v-env p-env c-env k-s k-ret))
        ((neg? exp)
	 (elog-neg exp v-env p-env c-env k-s k-ret))
        ((power? exp) 
	 (elog-power exp v-env p-env c-env k-s k-ret))
        ((syntax-equal? exp) 
	 (elog-syntax-equal exp v-env p-env c-env k-s k-ret))
        ((syntax-less? exp) 
	 (elog-syntax-less exp v-env p-env c-env k-s k-ret))
        ((syntax-greater? exp) 
	 (elog-syntax-greater exp v-env p-env c-env k-s k-ret))
        ((call? exp) 
	 (elog-call exp v-env p-env c-env k-s k-ret))
        (else (logo-error "unknown expression type" exp))))


(define (elog-output exp v-env p-env c-env k-s k-ret)
  (let ((exp1 (arg1 exp)))
    (elog exp1 v-env p-env c-env k-ret k-ret)))

(define (elog-logo-literal-word exp v-env p-env c-env k-s k-ret)
  (let* ((str (string<-logo-literal-word exp))
	 (rstr (substring str 1 (string-length str)))
	 (result (if (string=? rstr "")
		     (make-name "")
		     (car (logo-lex rstr)))))
    (k-s result)))


(define (elog-seq exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env 
	p-env 
	c-env
	(if (empty-seq? (arg2 exp))
	    (lambda (v) (k-s v))
	    (lambda (v)
	      (if (not (eq? v *proc-return*))
		  (logo-error "You don't say what to do with" v)
		  (elog (arg2 exp) v-env p-env c-env k-s k-ret))))
	k-ret))

(define (elog-catch exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (name)
	  (if (logo-name? name)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (rL)
		      (if (logo-list? rL)
			  (elog (parse rL) 
				v-env 
				p-env 
				(extend c-env
					(list (symbol<-logo-name name))
					(list k-s))
				k-s
				k-ret)
			  (logo-error 
			   "Catch's 2nd input should evaluate to a list")))
		    k-ret)
	      (logo-error "Catch's first input should evaluate to a name")))
	k-ret))

(define (elog-throw exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (name)
	  (if (logo-name? name)
	      (let ((new-k-s (c-look-up (symbol<-logo-name name) c-env)))
		(new-k-s *proc-return*))
	      (logo-error "Throw's input should be a word")))
	k-ret))

(define (elog-sum exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num1)
	  (if (number? num1)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (num2)
		      (if (number? num2)
			  (k-s (+ num1 num2))
			  (logo-error "Inputs to + should be numbers")))
		    k-ret)
	      (logo-error "Inputs to + should be numbers")))
	k-ret))

(define (elog-diff exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num1)
	  (if (number? num1)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (num2)
		      (if (number? num2)
			  (k-s (- num1 num2))
			  (logo-error "Inputs to - should be numbers")))
		    k-ret)
	      (logo-error "Inputs to - should be numbers")))
	k-ret))

(define (elog-prod exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num1)
	  (if (number? num1)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (num2)
		      (if (number? num2)
			  (k-s (* num1 num2))
			  (logo-error "Inputs to * should be numbers")))
		    k-ret)
	      (logo-error "Inputs to * should be numbers")))
	k-ret))

(define (elog-quo exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num1)
	  (if (number? num1)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (num2)
		      (if (number? num2)
			  (if (not (= num2 0))
			      (k-s (/ num1 num2))
			      (logo-error "Divide by zero error in /"))
			  (logo-error "Inputs to / should be numbers")))
		    k-ret)
	      (logo-error "Inputs to / should be numbers")))
	k-ret))

(define (elog-neg exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num)
	  (if (number? num)
	      (k-s (- num))
	      (logo-error "Input to - should be a number")))
	k-ret))

(define (elog-power exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num1)
	  (if (number? num1)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (num2)
		      (if (number? num2)
			  (if (or (and (zero? num1)
				       (real? num2)
				       (not (negative? num2)))
				  (not (zero? num1))
				  (not (real? num2)))
			      (k-s (expt num1 num2))
			      (logo-error "Divide by zero error in ^"))
			  (logo-error "Inputs to ^ should be numbers")))
		    k-ret)
	      (logo-error "Inputs to ^ should be numbers")))
	k-ret))

(define (elog-syntax-equal exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (e1)
	  (elog (arg2 exp)
		v-env
		p-env 
		c-env
		(lambda (e2)
		  (k-s (logo-bool<-bool (logo-equal? e1 e2))))
		k-ret))
	k-ret))

(define (elog-syntax-less exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num1)
	  (if (real? num1)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (num2)
		      (if (real? num2)
			  (k-s (if (< num1 num2) *logo-true* *logo-false*))
			  (logo-error "Inputs to < should be real numbers")))
		    k-ret)
	      (logo-error "Inputs to < should be real numbers")))
	k-ret))

(define (elog-syntax-greater exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env
	p-env 
	c-env
	(lambda (num1)
	  (if (real? num1)
	      (elog (arg2 exp)
		    v-env
		    p-env 
		    c-env
		    (lambda (num2)
		      (if (real? num2)
			  (k-s (if (> num1 num2) *logo-true* *logo-false*))
			  (logo-error "Inputs to > should be real numbers")))
		    k-ret)
	      (logo-error "Inputs to > should be real numbers")))
	k-ret))

(define (elog-if exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp) 
	v-env
	p-env 
	c-env
	(lambda (b)
	  (if (logo-boolean? b)
	      (if (bool<-logo-bool b)
		  (elog (arg2 exp) 
			v-env 
			p-env 
			c-env
			(lambda (rL) 
			  (if (logo-list? rL)
			      (elog (parse rL) v-env p-env c-env k-s k-ret)
			      (logo-error 
			       "If's 2nd input should be a list")))
			k-ret)
		  (k-s *proc-return*))
	      (logo-error "If's 1st input should be a Boolean")))
	k-ret))

(define (elog-ifelse exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp) 
	v-env
	p-env 
	c-env
	(lambda (b) 
	  (if (logo-boolean? b)
	      (if (bool<-logo-bool b)
		  (elog (arg2 exp) 
			v-env 
			p-env 
			c-env
			(lambda (rL) 
			  (if (logo-list? rL)
			      (elog (parse rL) v-env p-env c-env k-s k-ret)
			      (logo-error "Ifelse's 2nd input should be a list")))
			k-ret)
		  (elog (arg3 exp) 
			v-env 
			p-env 
			c-env
			(lambda (rL) 
			  (if (logo-list? rL)
			      (elog (parse rL) v-env p-env c-env k-s k-ret)
			      (logo-error "Ifelse's 3rd input should be a list")))
			k-ret))
	      (logo-error "Ifelse's 1st input should be a Boolean")))
	k-ret))

(define (elog-repeat exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env 
	p-env 
	c-env
	(lambda (num)
	  (if (and (number? num) (<= 0 num))
	      (elog (arg2 exp)
		    v-env 
		    p-env 
		    c-env
		    (lambda (rL)
		      (if (logo-list? rL)
			  (let ((run-exp (parse rL)))
			    (letrec ((k-rep
				      (lambda (v)
					(if (not (eq? v *proc-return*))
					    (logo-error 
					     "You don't say what to do with" v)
					    (begin
					      (set! num (- num 1))
					      (if (= num 0)
						  (k-s *proc-return*)
						  (elog run-exp
							v-env 
							p-env 
							c-env 
							k-rep
							k-ret)))))))
			      (if (= num 0)
				  (k-s *proc-return*)
				  (elog run-exp
					v-env 
					p-env 
					c-env 
					k-rep
					k-ret))))
			  (logo-error 
			   "Repeat's 2nd input should be a list")))
		    k-ret)
	      (logo-error 
	       "Repeat's 1st input should be a number")))
	k-ret))

(define (elog-make exp v-env p-env c-env k-s k-ret)
  (elog (arg1 exp)
	v-env 
	p-env 
	c-env
	(lambda (name)
	  (if (logo-name? name)
	      (elog (arg2 exp)
		    v-env 
		    p-env 
		    c-env
		    (lambda (val)
		      (update! v-env (symbol<-logo-name name) val)
		      (k-s *proc-return*))
		    k-ret)
	      (logo-error 
	       "Make expects its first input to be a variable name")))
	k-ret))

(define (elog-proc exp v-env p-env c-env k-s k-ret)
  (let* ((name (symbol<-logo-name (arg1 exp)))
	 (formals (map symbol<-logo-var (arg2 exp)))
	 (unparsed-body (arg3 exp))
	 (code (lambda (args v-env p-env c-env k-ret)
		 (elog (parse unparsed-body)
		       (extend v-env formals args)
		       p-env
		       c-env
		       k-ret
		       k-ret)))
	 (obj (make-user-proc code exp)))
    (update! p-env name obj)
    (k-s *proc-return*)))

(define (elog-call exp v-env p-env c-env k-s k-ret)
  (let* ((proc-entry (p-look-up (symbol<-logo-name (arg1 exp)) p-env))
	 (code (code<-logo-proc-entry proc-entry)))
    (if (and (logo-user-proc? proc-entry)
	     (< (length (arg2 exp)) (get-logo-arity (arg1 exp))))
	(logo-error (arg1 exp) "got too few inputs")
	(elog-args (arg2 exp)
		   v-env 
		   p-env 
		   c-env
		   (lambda (args) (code args v-env p-env c-env k-s))
		   k-ret))))

(define (elog-args exps v-env p-env c-env k-s k-ret)
  (if (null? exps)
    (k-s '())
    (elog (car exps) 
          v-env 
          p-env
          c-env
          (lambda (v)
            (if (eq? v *proc-return*)
              (logo-error "the caller was expecting a value")
              (elog-args (cdr exps)
                         v-env
                         p-env 
                         c-env
                         (lambda (L) (k-s (cons v L)))
                         k-ret)))
          k-ret)))
                           
(define (read-paragraph)
  (let loop ((p "")
             (s (read-line (current-input-port))))
    (if (= (string-length s) 0)
        p
        (loop (string-append p " " s)
               (read-line (current-input-port))))))

(define (logo)
  (display "?")
  (let* ((input (read-paragraph))
         (pinput (parse (logo-lex input)))
         (result
          (elog pinput *init-var-env* 
                       *init-proc-env* 
                       *init-catch-env*
                       (lambda (v) v) 
                       (lambda (v) v))))
    (if (eq? result *proc-return*)
      (logo)
      (logo-error "You don't say what to do with" result))))

(define *logo-reset-cont* 'dummy-val)

(define (logo-error . messages)
  (for-each (lambda (message)
              (if (or (number? message)
                      (tword? message)
                      (logo-list? message))
                (begin (logo-show message)
                       (display " "))
                (begin (display message)
                       (display " "))))
            messages)
  (newline)
  (logo-reset))


(define (logo-reset) (*logo-reset-cont* 'dummy-val))

(define (go-logo)
  (display "Welcome to Arthur Nunes-Harwitt's Classic Logo")
  (newline)
  (call-with-current-continuation
   (lambda (k)
     (set! *logo-reset-cont* k)))
  (logo))
