;; This work is licensed under the Creative Commons
;; Attribution-NonCommercial-ShareAlike License. To view a copy of this
;; license, visit http://creativecommons.org/licenses/by-nc-sa/2.0/ or
;; send a letter to Creative Commons, 559 Nathan Abbott Way, Stanford,
;; California 94305, USA.
;;
;; The original author of this code is Arthur Nunes-Harwitt
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;; Logo Primitives
;;;;;;;;;;;;;;;;;;;;;;;;;;



(define (every? p L)
  (if (null? L)
    #t
    (and (p (car L)) (every? p (cdr L)))))

(define (length-n? n L)
  (if (null? L)
    (= n 0)
    (length-n? (- n 1) (cdr L))))



(define (logo-eqv? x y)
  (or (and (null? x)
           (null? y))
      (and (number? x)
           (number? y)
           (= x y))
      (and (logo-var? x)
           (logo-var? y)
           (eq? (symbol<-logo-var x)
                (symbol<-logo-var y)))
      (and (logo-literal-word? x)
           (logo-literal-word? y)
           (eq? (symbol<-logo-literal-word x)
                (symbol<-logo-literal-word y)))
      (and (logo-name? x) 
           (logo-name? y) 
           (eq? (symbol<-logo-name x)
                (symbol<-logo-name y))) ))

(define (logo-equal? x y)
  (if (and (pair? x) (pair? y))
    (and (logo-equal? (car x) (car y))
         (logo-equal? (cdr x) (cdr y)))
    (logo-eqv? x y)))

(define (logo-equal* L)
  (if (null? L)
    #t
    (let loop ((item (car L))
               (rest (cdr L)))
      (if (null? rest)
        #t
        (if (logo-equal? item (car rest))
          (loop (car rest) (cdr rest))
          #f)))))

(define log-prim-equal 
  (lambda (args v-env p-env c-env k-ret) 
    (k-ret (logo-bool<-bool(logo-equal* args)))))

(define (log-prim-and* L)
  (cond ((null? L) *logo-true*)
        ((logo-true? (car L)) (log-prim-and* (cdr L)))
        ((logo-false? (car L)) *logo-false*)
        (else (logo-error "And expects Boolean inputs"))))

(define log-prim-and 
  (lambda (args v-env p-env c-env k-ret) 
    (k-ret (log-prim-and* args))))

(define (log-prim-or* L)
  (cond ((null? L) *logo-false*)
        ((logo-true? (car L)) *logo-true*)
        ((logo-false? (car L)) (log-prim-or* (cdr L)))
        (else (logo-error "Or expects Boolean inputs"))))

(define log-prim-or
  (lambda (args v-env p-env c-env k-ret) 
    (k-ret (log-prim-or* args))))

(define log-prim-not
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (cond ((logo-true? (car args)) (k-ret *logo-false*))
            ((logo-false? (car args)) (k-ret *logo-true*))
            (else (logo-error "Not expects a Boolean input")))
      (logo-error "Not got the wrong number of inputs"))))
    

(define log-prim-listp
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (if (logo-list? (car args)) 
        (k-ret *logo-true*)
        (k-ret *logo-false*))
      (logo-error "ListP got the wrong number of inputs"))))

(define log-prim-numberp
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (if (number? (car args)) 
        (k-ret *logo-true*)
        (k-ret *logo-false*))
      (logo-error "NumberP got the wrong number of inputs"))))

(define (log-prim-logo-word? x)
  (or (number? x) (tword? x)))


(define log-prim-wordp
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (if (log-prim-logo-word? (car args))
        (k-ret *logo-true*)
        (k-ret *logo-false*))
      (logo-error "WordP got the wrong number of inputs"))))

(define log-prim-emptyp
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (if (or (null? (car args))
              (and (tword? (car args))
                   (string=? (string<-tword (car args)) "")))
        (k-ret *logo-true*)
        (k-ret *logo-false*))
      (logo-error "EmptyP got the wrong number of inputs"))))

(define (log-prim-prod* L)
  (if (null? L) 
    1
    (if (number? (car L))
      (* (car L) (log-prim-prod* (cdr L)))
      (logo-error "Product's inputs should be numbers"))))

(define log-prim-prod 
  (lambda (args v-env p-env c-env k-ret) (k-ret (log-prim-prod* args))))

(define (log-prim-sum* L)
  (if (null? L) 
    0
    (if (number? (car L))
      (+ (car L) (log-prim-sum* (cdr L)))
      (logo-error "Sum's inputs should be numbers"))))


(define log-prim-sum 
  (lambda (args v-env p-env c-env k-ret) (k-ret (log-prim-sum* args))))

(define log-prim-diff 
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 2 args)
      (if (and (number? (car args))
               (number? (cadr args)))
        (k-ret (- (car args) (cadr args)))
        (logo-error "Difference's inputs should be numbers"))
      (logo-error "Difference got the wrong number of inputs"))))

(define log-prim-quot 
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 2 args)
      (if (and (number? (car args))
               (number? (cadr args))
               (not (= 0 (cadr args))))
        (k-ret (/ (car args) (cadr args)))
        (logo-error "Quotient's inputs should be a number and a non-zero number"))
      (logo-error "Quotient got the wrong number of inputs"))))

(define log-prim-power
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 2 args)
      (if (and (number? (car args))
               (number? (cadr args)))
        (if (or (and (zero? (car args)) 
                     (real? (cadr args))
                     (not (negative? (cadr args))))
                (not (zero? (car args)))
                (not (real? (cadr args))))
          (k-ret (expt (car args) (cadr args)))
          (logo-error "Divide by zero error in Power"))
        (logo-error "Power's inputs should be numbers"))
      (logo-error "Power got the wrong number of inputs"))))

(define log-prim-minus
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (if (number? (car args))
        (k-ret (- (car args)))
        (logo-error "Minus's inputs should be numbers"))
      (logo-error "Minus got the wrong number of inputs"))))


(define log-prim-gt
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 2 args)
      (if (and (real? (car args))
               (real? (cadr args)))
        (k-ret (logo-bool<-bool (> (car args) (cadr args))))
        (logo-error "GreaterP's inputs should be real numbers"))
      (logo-error "GreaterP got the wrong number of inputs"))))

(define log-prim-lt
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 2 args)
      (if (and (real? (car args))
               (real? (cadr args)))
        (k-ret (logo-bool<-bool (< (car args) (cadr args))))
        (logo-error "LessP's inputs should be real numbers"))
      (logo-error "LessP got the wrong number of inputs"))))


(define log-prim-list
  (lambda (args v-env p-env c-env k-ret) (k-ret args)))

(define (log-prim-se2 x y)
  (cond ((and (logo-list? x)
              (logo-list? y))
         (append x y))
        ((and (log-prim-logo-word? x)
              (log-prim-logo-word? y))
         (list x y))
        ((and (log-prim-logo-word? x)
              (logo-list? y))
         (cons x y))
        ((and (logo-list? x)
              (log-prim-logo-word? y))
         (append x (list y)))
        (else (logo-error "Bad type combination for se" x y))))

(define (log-prim-se* L)
  (if (null? L)
    '()
    (let loop ((hd (car L))
               (rest (cdr L)))
      (if (null? rest)
        (cond ((logo-list? hd) hd)
              ((log-prim-logo-word? hd) (list hd))
              (else (logo-error "Bad type for se" hd)))
        (if (null? (cdr rest))
          (log-prim-se2 hd (car rest))
          (log-prim-se2 hd (loop (car rest) (cdr rest))))))))

(define log-prim-se
  (lambda (args v-env p-env c-env k-ret) (k-ret (log-prim-se* args))))

(define (log-prim-first1 x)
  (cond ((pair? x) (car x))
        ((tword? x)
         (if (string=? (string<-tword x) "")
           (logo-error "First doesn't like the empty word")
           (car (logo-lex (substring (string<-tword x) 0 1)))))
        ((number? x) (car (logo-lex (substring (number->string x) 0 1))))
        (else (logo-error "First doesn't like the input" x))))
 
(define log-prim-first
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (k-ret (log-prim-first1 (car args)))
      (logo-error "First got the wrong number of inputs"))))

(define (log-prim-bf1 x)
  (cond ((pair? x) (cdr x))
        ((tword? x)
         (if (string=? (string<-tword x) "")
           (logo-error "Butfirst doesn't like the empty word")
           (let* ((str (string<-tword x))
                  (rest (substring str 1 (string-length str))))
             (if (string=? rest "")
               (make-name "")
               (car (logo-lex rest))))))
        ((number? x) 
         (let* ((nstr (number->string x))
                (rest (substring nstr 1 (string-length nstr))))
           (if (string=? rest "")
             (make-name "")
             (car (logo-lex rest)))))
        (else (logo-error "Butfirst doesn't like the input" x))))

(define log-prim-bf
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (k-ret (log-prim-bf1 (car args)))
      (logo-error "Butfirst got the wrong number of inputs"))))


(define (log-prim-last1 x)
  (cond ((pair? x) (car (reverse x)))
        ((tword? x)
         (let ((str (string<-tword x)))
           (if (string=? str "")
             (logo-error "Last doesn't like the empty word")
             (let ((len (string-length str)))
               (car (logo-lex (substring str (- len 1) len)))))))
        ((number? x) 
         (let* ((nstr (number->string x))
                (len (string-length nstr)))
           (car (logo-lex (substring nstr (- len 1) len)))))
        (else (logo-error "Last doesn't like the input" x))))
 
(define log-prim-last
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (k-ret (log-prim-last1 (car args)))
      (logo-error "Last got the wrong number of inputs"))))

(define (log-prim-bl1 x)
  (cond ((pair? x) (reverse (cdr (reverse x))))
        ((tword? x)
         (let* ((str (string<-tword x))
                (len (string-length str)))
           (if (= len 0)
             (logo-error "Butlast doesn't like the empty word")
             (let ((rest (substring str 0 (- len 1))))
               (if (string=? rest "")
                 (make-name "")
                 (car (logo-lex rest)))))))
        ((number? x) 
         (let* ((nstr (number->string x))
                (rest (substring nstr 0 (- (string-length nstr) 1))))
           (if (string=? rest "")
             (make-name "")
             (car (logo-lex rest)))))
        (else (logo-error "Butlast doesn't like the input" x))))

(define log-prim-bl
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (k-ret (log-prim-bl1 (car args)))
      (logo-error "Butlast got the wrong number of inputs"))))

(define (log-prim-count1 x)
  (cond ((logo-list? x) (length x))
        ((tword? x) (string-length (string<-logo-name x)))
        ((number? x) (string-length (number->string x)))
        (else (logo-error "Count doesn't like the input" x))))

(define log-prim-count
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (k-ret (log-prim-count1 (car args)))
      (logo-error "Count got the wrong number of inputs"))))


(define log-prim-fput
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 2 args)
      (if (logo-list? (cadr args))
        (k-ret (cons (car args) (cadr args)))
        (logo-error "Fput's second input must be a list"))
      (logo-error "Fput got the wrong number of inputs"))))


(define log-prim-lput
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 2 args)
      (if (logo-list? (cadr args))
        (k-ret (reverse (cons (car args) (reverse (cadr args)))))
        (logo-error "Lput's second input must be a list"))
      (logo-error "Lput got the wrong number of inputs"))))


(define log-prim-item
  (lambda (args v-env p-env c-env k-ret)
    (if (length-n? 2 args)
      (let ((n (car args))
            (woL (cadr args)))
        (if (and (integer? n) (<= 1 n))
          (cond ((logo-list? woL)
                 (let loop ((n (- n 1))
                            (L woL))
                   (cond ((null? L) 
                          (logo-error "The input list to Item is too short"))
                         ((= n 0) (k-ret (car L)))
                         (else (loop (- n 1) (cdr L))))))
                ((tword? woL) 
                 (let ((str (string<-tword woL)))
                   (if (< (string-length str) n)
                     (logo-error "The input word to Item is too short")
                     (k-ret (car (logo-lex 
                                  (make-string 1 (string-ref str (- n 1)))))))))
                ((number? woL)
                 (let ((str (number->string woL)))
                   (if (< (string-length str) n)
                     (logo-error "The input word to Item is too short")
                     (k-ret (car (logo-lex 
                                  (make-string 1 (string-ref str (- n 1)))))))))
                (else (logo-error "Item's second input should be a word or a list")))
          (logo-error "Item's first input should be a whole number")))
      (logo-error "Item got the wrong number of inputs"))))
                   

(define (string-member? k s)
  (let loop ((n (- (string-length s) 1)))
    (cond ((< n 0) #f)
          ((char=? k (string-ref s n)) #t)
          (else (loop (- n 1))))))
      

(define log-prim-memberp
  (lambda (args v-env p-env c-env k-ret)
    (if (length-n? 2 args)
      (let ((a1 (car args))
            (a2 (cadr args)))
        (cond ((logo-list? a2)
               (let loop ((L a2))
                 (cond ((null? L) (k-ret *logo-false*))
                       ((logo-equal? a1 (car L))
                        (k-ret *logo-true*))
                       (else (loop (cdr L))))))
              ((tword? a2)
               (cond ((and (tword? a1)
                           (= 1 (string-length (string<-tword a1))))
                      (k-ret (logo-bool<-bool 
                              (string-member? 
                               (string-ref (string<-tword a1) 0)
                               (string<-tword a2)))))
                     ((number? a1)
                      (let ((str (number->string a1)))
                        (if (= 1 (string-length str)) 
                          (k-ret (logo-bool<-bool 
                                  (string-member? 
                                   (string-ref str 0)
                                   (string<-tword a2))))
                          (logo-error "MemberP's first input should be a count one word"))))
                     (else (logo-error "MemberP's first input should be a count one word"))))
              ((number? a2)
               (cond ((and (tword? a1)
                           (= 1 (string-length (string<-tword a1))))
                      (k-ret (logo-bool<-bool 
                              (string-member? 
                               (string-ref (string<-tword a1) 0) 
                               (number->string a2)))))
                     ((number? a1)
                      (let ((str (number->string a1)))
                        (if (= 1 (string-length str)) 
                          (k-ret (logo-bool<-bool 
                                  (string-member? 
                                   (string-ref str 0)
                                   (number->string a2))))
                          (logo-error "MemberP's first input should be a count one word"))))
                     (else (logo-error "MemberP's first input should be a count one word"))))
              (else (logo-error "MemberP's second input should be a word or list"))) )
      (logo-error "MemberP got the wrong number of inputs"))))


(define log-prim-ascii
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (let ((arg (car args)))
        (cond ((and (tword? arg)
                    (= 1 (string-length (string<-tword arg))))
               (k-ret (char->integer (string-ref (string<-tword arg) 0))))
              ((number? arg)
               (let ((str (number->string arg)))
                 (if (= 1 (string-length str))
                   (k-ret (char->integer (string-ref str 0)))
                   (logo-error "Ascii's input must be a count one word"))))
              (else (logo-error "Ascii's input must be a count one word"))))
      (logo-error "Ascii got the wrong number of inputs"))))


(define log-prim-char
  (lambda (args v-env p-env c-env k-ret) 
    (if (length-n? 1 args)
      (let ((arg (car args)))
        (if (and (integer? arg)
                 (<= 0 arg)
                 (<= arg 255))
          (k-ret (car (logo-lex (make-string 1 (integer->char arg)))))
          (logo-error "Char's input must be an integer between 0 and 255")))
      (logo-error "Char got the wrong number of inputs"))))


(define (log-prim-word2 x y)
  (cond ((and (tword? x) (tword? y))
         (car (logo-lex (string-append (string<-tword x) (string<-tword y)))))
        ((and (tword? x) (number? y))
         (car (logo-lex (string-append (string<-tword x) (number->string y)))))
        ((and (number? x) (tword? y))
         (car (logo-lex (string-append (number->string x) (string<-tword y)))))
        ((and (number? x) (number? y))
         (car (logo-lex (string-append (number->string x) (number->string y)))))
        (else (logo-error "Word's inputs must be words"))))

(define (log-prim-word* L)
  (if (null? L)
    (make-name "")
    (log-prim-word2 (car L) (log-prim-word* (cdr L)))))

(define log-prim-word
  (lambda (args v-env p-env c-env k-ret) (k-ret (log-prim-word* args))))

(define (logo-show exp)
  
  (define (show-logo-list exp)
    (cond ((null? exp) (display "]"))
          ((null? (cdr exp))
           (logo-show (car exp))
           (display "]"))
          (else (logo-show (car exp))
                (display " ")
                (show-logo-list (cdr exp)))))
           

  (cond ((logo-list? exp)
         (display "[")
         (show-logo-list exp))
        ((number? exp)
         (display exp))
        ((tword? exp)
         (display (string<-tword exp)))
        (else (logo-error "Unknown expression type" exp))))

(define (logo-show* L)
  (cond ((null? L) 'done)
        ((null? (cdr L)) (logo-show (car L)))
        (else (logo-show (car L))
              (display " ")
              (logo-show* (cdr L)))))

(define log-prim-show 
  (lambda (args v-env p-env c-env k-ret) (logo-show* args)
                                         (newline)
                                         (k-ret *proc-return*)))


(define (log-prim-print1 exp)
  (if (logo-list? exp)
    (let loop ((L exp))
      (cond ((null? L) 'done)
            ((null? (cdr L)) (logo-show (car L)))
            (else (logo-show (car L))
                  (display " ")
                  (loop (cdr L)))))
    (logo-show exp)))

(define (log-prim-print* L)
  (cond ((null? L) 'done)
        ((null? (cdr L)) (log-prim-print1 (car L)))
        (else (log-prim-print1 (car L))
              (display " ")
              (log-prim-print* (cdr L)))))

(define log-prim-print
  (lambda (args v-env p-env c-env k-ret) (log-prim-print* args)
                                         (newline)
                                         (k-ret *proc-return*)))

(define (log-prim-type* L)
  (if (null? L) 
    'done
    (begin (log-prim-print1 (car L))
           (log-prim-type* (cdr L)))))

(define log-prim-type 
  (lambda (args v-env p-env c-env k-ret) (log-prim-type* args)
                                         (k-ret *proc-return*)))


(define log-prim-readlist
  (lambda (args v-env p-env c-env k-ret)
    (if (length-n? 0 args)
      (k-ret (logo-lex (read-line (current-input-port))))
      (logo-error "Readlist got too many inputs"))))

(define (log-prim-text1 name v-env p-env c-env k-ret)
  (if (logo-name? name)
    (let ((proc-entry (p-look-up (symbol<-logo-name name) p-env)))
      (if (logo-user-proc? proc-entry)
        (let ((names (map (lambda (v)
                            (logo-name<-symbol (symbol<-logo-var v)))
                          (arg2 (rep<-user-proc proc-entry)))))
          (k-ret (cons names 
                       (list (arg3 (rep<-user-proc proc-entry))))))
        (logo-error "Primitive procedures have no text")))
    (logo-error "Text's input should be a name")))

(define log-prim-text
  (lambda (args v-env p-env c-env k-ret)
    (if (length-n? 1 args)
      (log-prim-text1 (car args) v-env p-env c-env k-ret)
      (logo-error "Text got the wrong number of inputs"))))


(define (log-prim-define2 name def-list v-env p-env c-env k-ret)
  (if (logo-name? name)
    (if (and (pair? def-list)
             (every? logo-list? def-list)
             (or (pair? (car def-list)) (null? (car def-list)))
             (every? logo-name? (car def-list))
             (pair? (cdr def-list)))
      (let ((vars (map (lambda (n)
                            (logo-var<-symbol (symbol<-logo-name n)))
                          (car def-list)))
            (formals (map (lambda (n) (symbol<-logo-name n))
                          (car def-list)))
            (unparsed-body (apply append (cdr def-list))))
        (update! p-env
                 (symbol<-logo-name name)
                 (make-user-proc (lambda (args v-env p-env c-env k-ret)
                                   (elog (parse unparsed-body)
                                         (extend v-env formals args)
                                         p-env
                                         c-env
                                         (lambda (v) v)
                                         k-ret))
                                 (make-proc name vars unparsed-body)))
        (put-logo-arity name (length vars))
        (k-ret *proc-return*))
      (logo-error "Badly formed definition list"))
    (logo-error "Define's 1st input should be a name")))

(define log-prim-define
  (lambda (args v-env p-env c-env k-ret)
    (if (length-n? 2 args)
      (log-prim-define2 (car args) (cadr args) v-env p-env c-env k-ret)
      (logo-error "Define got the wrong number of inputs"))))
        

(define (logo-prim-run1 rL v-env p-env c-env k-ret)
  (if (logo-list? rL)
    (elog (parse rL)
          v-env 
          p-env 
          c-env
          k-ret
          k-ret)
    (logo-error "Run expects its input to be a list")))

(define logo-prim-run
  (lambda (args v-env p-env c-env k-ret)
    (if (length-n? 1 args)
      (logo-prim-run1 (car args) v-env p-env c-env k-ret)
      (logo-error "Run got the wrong number of inputs"))))

(define (logo-prim-bye args v-env p-env c-env k-ret)
  (reset) )
              
(put-logo-arity (logo-name<-symbol 'equalp) 2)
(put-logo-arity (logo-name<-symbol 'and) 2)
(put-logo-arity (logo-name<-symbol 'or) 2)
(put-logo-arity (logo-name<-symbol 'not) 1)
(put-logo-arity (logo-name<-symbol 'listp) 1)
(put-logo-arity (logo-name<-symbol 'wordp) 1)
(put-logo-arity (logo-name<-symbol 'numberp) 1)
(put-logo-arity (logo-name<-symbol 'emptyp) 1)
(put-logo-arity (logo-name<-symbol 'product) 2)
(put-logo-arity (logo-name<-symbol 'sum) 2)
(put-logo-arity (logo-name<-symbol 'difference) 2)
(put-logo-arity (logo-name<-symbol 'quotient) 2)
(put-logo-arity (logo-name<-symbol 'power) 2)
(put-logo-arity (logo-name<-symbol 'minus) 1)
(put-logo-arity (logo-name<-symbol 'greaterp) 2)
(put-logo-arity (logo-name<-symbol 'lessp) 2)
(put-logo-arity (logo-name<-symbol 'list) 2)
(put-logo-arity (logo-name<-symbol 'sentence) 2)
(put-logo-arity (logo-name<-symbol 'se) 2)
(put-logo-arity (logo-name<-symbol 'first) 1)
(put-logo-arity (logo-name<-symbol 'butfirst) 1)
(put-logo-arity (logo-name<-symbol 'bf) 1)
(put-logo-arity (logo-name<-symbol 'last) 1)
(put-logo-arity (logo-name<-symbol 'butlast) 1)
(put-logo-arity (logo-name<-symbol 'bl) 1)
(put-logo-arity (logo-name<-symbol 'count) 1)
(put-logo-arity (logo-name<-symbol 'fput) 2)
(put-logo-arity (logo-name<-symbol 'lput) 2)
(put-logo-arity (logo-name<-symbol 'show) 1)
(put-logo-arity (logo-name<-symbol 'item) 2)
(put-logo-arity (logo-name<-symbol 'memberp) 2)
(put-logo-arity (logo-name<-symbol 'ascii) 1)
(put-logo-arity (logo-name<-symbol 'char) 1)
(put-logo-arity (logo-name<-symbol 'word) 2)
(put-logo-arity (logo-name<-symbol 'type) 1)
(put-logo-arity (logo-name<-symbol 'print) 1)
(put-logo-arity (logo-name<-symbol 'readlist) 0)
(put-logo-arity (logo-name<-symbol 'rl) 0)
(put-logo-arity (logo-name<-symbol 'text) 1)
(put-logo-arity (logo-name<-symbol 'define) 2)
(put-logo-arity (logo-name<-symbol 'run) 1)
(put-logo-arity (logo-name<-symbol 'bye) 0)

(define *init-proc-env* (extend '() '(equalp 
                                      and
                                      or
                                      not
                                      listp
                                      wordp
                                      numberp
                                      emptyp
                                      product 
                                      sum 
                                      difference 
                                      quotient
                                      power
                                      minus
                                      greaterp
                                      lessp
                                      list
                                      sentence
                                      se
                                      first
                                      butfirst
                                      bf
                                      last
                                      butlast
                                      bl
                                      count
                                      fput
                                      lput
                                      show
                                      item
                                      memberp
                                      ascii
                                      char
                                      word
                                      type
                                      print
                                      readlist
                                      rl
                                      text
                                      define
                                      run
				      bye
                                      )
                                    (list (make-primitive log-prim-equal)
                                          (make-primitive log-prim-and)
                                          (make-primitive log-prim-or)
                                          (make-primitive log-prim-not)
                                          (make-primitive log-prim-listp)
                                          (make-primitive log-prim-wordp)
                                          (make-primitive log-prim-numberp)
                                          (make-primitive log-prim-emptyp)
                                          (make-primitive log-prim-prod)
                                          (make-primitive log-prim-sum)
                                          (make-primitive log-prim-diff)
                                          (make-primitive log-prim-quot)
                                          (make-primitive log-prim-power)
                                          (make-primitive log-prim-minus)
                                          (make-primitive log-prim-gt)
                                          (make-primitive log-prim-lt)
                                          (make-primitive log-prim-list)
                                          (make-primitive log-prim-se)
                                          (make-primitive log-prim-se)
                                          (make-primitive log-prim-first)
                                          (make-primitive log-prim-bf)
                                          (make-primitive log-prim-bf)
                                          (make-primitive log-prim-last)
                                          (make-primitive log-prim-bl)
                                          (make-primitive log-prim-bl)
                                          (make-primitive log-prim-count)
                                          (make-primitive log-prim-fput)
                                          (make-primitive log-prim-lput)
                                          (make-primitive log-prim-show)
                                          (make-primitive log-prim-item)
                                          (make-primitive log-prim-memberp)
                                          (make-primitive log-prim-ascii)
                                          (make-primitive log-prim-char)
                                          (make-primitive log-prim-word)
                                          (make-primitive log-prim-type)
                                          (make-primitive log-prim-print)
                                          (make-primitive log-prim-readlist)
                                          (make-primitive log-prim-readlist)
                                          (make-primitive log-prim-text)
                                          (make-primitive log-prim-define)
                                          (make-primitive logo-prim-run)
					  (make-primitive logo-prim-bye)
                                          )))



